package mi.hdm.controllers;

import javafx.fxml.FXML;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextField;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import mi.hdm.components.IngredientSearchResultLabel;
import mi.hdm.recipes.*;
import mi.hdm.shoppingList.ShoppingList;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.IntStream;

public class ShoppingListViewController extends BaseController {
    private static final Logger log = LogManager.getLogger(ShoppingListViewController.class);

    private static final int ELEMENTS_PER_SEARCH_PAGE = 100;
    private final RecipeManager recipeManager;
    private final ShoppingList shoppingList;
    private final IngredientManager ingredientManager;
    private final RecipeSearch recipeSearch;
    private List<RecipeComponent> searchResults;
    private final List<RecipeComponent> selectedIngredients;


    private int currentPage = 0;
    private int maxPages = 0;

    @FXML
    private AnchorPane parent;
    @FXML
    private TextField ingredientSearch;
    @FXML
    private ScrollPane selectedIngredientsScrollPane;
    @FXML
    private VBox selectedIngredientsVbox;
    @FXML
    private ScrollPane searchResultsPane;



    public ShoppingListViewController(RecipeManager recipeManager, ShoppingList shoppingList, IngredientManager ingredientManager) {
        this.recipeManager = recipeManager;
        this.shoppingList = shoppingList;
        this.ingredientManager = ingredientManager;

        searchResults = new ArrayList<>();
        selectedIngredients = new ArrayList<>();
        recipeSearch = new RecipeSearch(recipeManager, ingredientManager);
        selectedIngredientsVbox = new VBox();
    }

    @FXML
    public void changeSceneToMain() {
        try {
            changeScene(View.MAIN);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("Something went wrong when changing the scene.");
        }
    }


    @FXML
    public void initialize() {
        loadHeader(parent);
        displayShoppingList();
        ingredientSearch.textProperty().addListener(((e) -> searchIngredients())); //adds onChange listener to the search field
        selectedIngredientsScrollPane.setContent(selectedIngredientsVbox);
    }

    @FXML
    public void handleRemoveBought() {
        shoppingList.removeAllBought();
        displayShoppingList();
    }
    @FXML
    public void handleClearList() {
        shoppingList.clear(); //no logging because logging is in method clear()
        displayShoppingList();
    }

    @FXML
    public void searchIngredients() {
        String input = ingredientSearch.getText();
        log.debug("Input in search field changed, searching through recipe components with query '{}'.", input);
        searchResults = recipeSearch.searchThroughNames(input);
        currentPage = 0;
        maxPages = searchResults.size() / ELEMENTS_PER_SEARCH_PAGE;
        drawIngredientSearchResults();
    }

    private void drawIngredientSearchResults() {
        log.debug("Drawing search results");
        VBox resultContainer = new VBox();
        int start = currentPage * ELEMENTS_PER_SEARCH_PAGE;
        int end = Math.min(searchResults.size(), start + ELEMENTS_PER_SEARCH_PAGE);
        IntStream.range(start, end)
                .mapToObj(searchResults::get)
                .forEach(
                        result -> {
                            IngredientSearchResultLabel resultLabel = new IngredientSearchResultLabel(result);
                            resultLabel.setOnMouseClicked(e -> {
                                log.debug("User added ingredient '{}' to recipe.", result.getName());
                                if (!selectedIngredients.contains(result)) {
                                    if (result instanceof Recipe r) {
                                        shoppingList.addAllToShoppingList(r);
                                    } else {
                                        selectedIngredients.add(result);
                                        shoppingList.addToShoppingList(result.getUniqueCode());
                                    }
                                    displayShoppingList();
                                }
                            });
                            resultContainer.getChildren().add(resultLabel);
                        }
                );
        searchResultsPane.setContent(resultContainer);
    }

    /**
     * This method reloads the ingredients in the shopping list view
     * by clearing it and then refill it with all ingredients from the current shoppingList.
     * If an ingredient is marked as done, it is loaded with a ticked checkbox.
     */
    private void displayShoppingList() {
        selectedIngredientsVbox.getChildren().clear();
        shoppingList.getList().forEach((code, status) -> {
            Ingredient i = ingredientManager.getIngredient(code).orElseThrow();
            selectedIngredients.add(i);
            CheckBox check = new CheckBox();
            check.setStyle(" -fx-padding: 5");
            Label name = new Label(i.getName());
            name.setStyle("-fx-padding: 5");
            selectedIngredientsVbox.getChildren().add(new HBox(check, name));
            if (shoppingList.getList().get(i.getUniqueCode())) { //if the ingredient is marked as done, the checkbox is ticked
                check.setSelected(true);
            }
            check.setOnMouseClicked(e -> {
                shoppingList.flipStatus(i);
            });
        });
    }
}
