package mi.hdm.recipes;

import mi.hdm.exceptions.InvalidCategoryException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

public class CategoryManager {
    private static final Logger log = LogManager.getLogger(CategoryManager.class);

    private final Map<Integer, Category> allCategories;

    public CategoryManager() {
        allCategories = new HashMap<>();
        log.info("Created empty instance of CategoryManager");
    }

    @Deprecated
    public CategoryManager(Map<Integer, Category> categories) {
        allCategories = new HashMap<>(categories);
        log.info("Created instance of CategoryManager with {} categories.", categories.size());
    }

    public CategoryManager(List<Category> categories) {
        allCategories = new HashMap<>();
        categories.forEach(c -> allCategories.put(c.getCode(), c));
    }

    /**
     * Adds a category if neither the given name nor the color code have been used in a category before.
     *
     * @param name       Name for the category that is to be added
     * @param colourCode Color code for the category that is to be added
     * @throws InvalidCategoryException if the category already exists
     */
    public void addCategory(String name, int colourCode) {
        Category c = new Category(name, colourCode);
        addCategory(c);
    }

    public void addCategory(Category c) {
        if (allCategories.get(c.getCode()) != null || getCategoryByName(c.getName()).isPresent() || getCategoryByColourCode(c.getColourCode()).isPresent()) {
            log.error("Category '{}' not added because it already exists", c.getName());
            throw new InvalidCategoryException("Category already exists.");
        } else {
            allCategories.put(c.getCode(), c);
            log.info("Category '{}' added successfully.", c.getName());
        }
    }

    public void deleteCategory(Category c) {
        if (!allCategories.values().remove(c)) {
            throw new InvalidCategoryException("Category is not listed.");
        }
        log.info("Category '{}' deleted successfully.", c.getName());
    }

    public void deleteCategory(String name) {
        Category c = getCategoryByName(name).orElseThrow(() -> new InvalidCategoryException("Category not found."));
        deleteCategory(c);
    }

    public List<Category> getAllCategories() {
        return allCategories.values().stream().toList();
    }

    public void clearCategories() {
        allCategories.clear();
        log.info("List of categories cleared successfully.");
    }

    private Optional<Category> getCategoryByName(String name) {
        return allCategories.values().stream()
                .filter(c -> c.getName().equals(name))
                .findFirst();
    }

    private Optional<Category> getCategoryByColourCode(String colourCode) {
        return allCategories.values().stream()
                .filter(c -> c.getColourCode().equals(colourCode))
                .findFirst();
    }

    public Optional<Category> get(Integer key) {
        return Optional.ofNullable(allCategories.get(key));
    }

    public List<Category> getCategoriesFromKeys(List<Integer> keys) {
        return keys
                .stream()
                .map(key -> {
                    Category c = allCategories.get(key);
                    if (c == null)
                        throw new InvalidCategoryException("Invalid category: no category exists with key " + key);
                    return c;
                })
                .toList();
    }

    public static String numberToColorCodeString(int colourCode) {
        //TODO: move method into Category class, remove the #
        String intToHex = Integer.toHexString(colourCode);
        intToHex = intToHex
                .indent(6 - intToHex.length())
                .replace(" ", "0");
        return ("#" + intToHex).strip();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o instanceof CategoryManager c) {
            return c.getAllCategories().equals(this.getAllCategories());
        }
        return false;
    }
}
