package mi.hdm.recipes;

import mi.hdm.exceptions.InvalidRecipeException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

public class RecipeManager {
    private static final Logger log = LogManager.getLogger(RecipeManager.class);

    private final Map<String, Recipe> allRecipes;

    public RecipeManager(List<Recipe> recipes) {
        this.allRecipes = new HashMap<>();
        recipes.forEach(i -> allRecipes.put(i.getName(), i));
        log.info("Loaded recipe manager with {} recipes", allRecipes.keySet().size());
    }

    public RecipeManager() {
        allRecipes = new HashMap<>();
        log.info("Created empty instance of RecipeManager");
    }

    public void addRecipe(Recipe recipe) {
        if (getRecipeByName(recipe.getName()).isPresent()) {
            log.error("Recipe '{}' not added because another recipe with the same name already exists", recipe.getName());
            throw new InvalidRecipeException("Recipe with this name already exists.");
        }
        allRecipes.put(recipe.getUniqueCode(), recipe);
        log.info("Recipe '{}' added successfully.", recipe.getName());
    }

    public Recipe getRecipeByCode(String code) {
        return allRecipes.get(code);
    }

    public void deleteRecipe(String name) {
        Recipe r = getRecipeByName(name).orElseThrow(() -> new InvalidRecipeException("A recipe with this name does not exist"));
        log.info("Recipe '{}' deleted successfully.", name);
        allRecipes.values().remove(r);
    }

    // Exception vs return false: What's the best approach?
    // -> false for normal situations, exceptions for real faults which should not occur
    public void deleteRecipe(Recipe r) {
        if (!allRecipes.values().remove(r)) {
            throw new InvalidRecipeException("Recipe is not listed.");
        }
        log.info("Recipe deleted successfully.");
    }

    private Optional<Recipe> getRecipeByName(String name) {
        for (final Recipe r : allRecipes.values()) {
            if (name.equals(r.getName())) {
                return Optional.of(r);
            }
        }
        return Optional.empty();
    }

    public void clear() {
        allRecipes.clear();
    }

    public List<Recipe> getRecipes() {
        return allRecipes.values().stream().toList();
    }
}
